//SlapJack

//Goal: beat the player by rotating 180 degrees before the player can pull their hands away,
// player can earn points by pulling their hands away before the arm makes a full turn. Computer
// can also earn points by making the player pull their hands away on a feint

//proximity sensor checks if players hands are in place
//game mode determined by potentiometer position (off, easy, medium, hard)
//computer can either feint to a random angle or make a full turn (harder game mode means computer is more likely to feint)
//computer moves faster at harder game modes

//first to earn 3 points wins, game resets after gameover LED sequence

#include <Servo.h>

Servo servo1;
const int trigger = 8;
const int echo = 7;    

float dist;
int speed; //set game mode (speed)
int outofrange = 8; //approx distance between US sensor and player's hands

int computerscore = 0;
int playerscore = 0;

void setup()
{
  Serial.begin(9600);
  
  //Servo: signal to pin 2, pulse width limits (500,2500)
  servo1.attach(2,500,2500);
  
  //Distance Sensor: input - 7, output - 8
  pinMode(trigger,OUTPUT);
  pinMode(echo,INPUT);
  
  //LED (red-computer)
  pinMode(13,OUTPUT);
  pinMode(12,OUTPUT);
  pinMode(11,OUTPUT);
  
  //LED (blue-player)
  pinMode(6,OUTPUT);
  pinMode(5,OUTPUT);
  pinMode(4,OUTPUT);

  servo1.write(0); //always start at 0
  Serial.println("setup complete");
}

void loop()
{

  int mode = analogRead(A0); //potentiometer gives number between 0-1023
  
  //Off
  if (mode < 255)
    speed = 0;
  //Easy Mode
  else if(mode < 511){
    speed = 3;
  }
  //Medium Mode
  else if(mode < 765){
    speed = 2;
  }
  //Hard Mode
  else{   
    speed = 1;
  }
  Serial.println(speed);
  
  //choose strategy based on game mode if player is ready (hard mode more likely to feint)
  if (speed!=0 && inRange()){
    Serial.println("play");
    int randstrat = random(11);
    if (randstrat <= speed+3){
      fullturn(speed);
      Serial.println("full");
    }
    else{
      feint(speed);
      Serial.println("feint");
    }
  }
  //time delay before next move (1-2.5 sec)
  delay(500*random(2,6));
}

void feint(int speed){
  //start from 0, turn to final angle at game speed, check for hands, return to 0
  
  //generate random final angle position between 0 and 100 (incomplete rotation)
  int ran = random(0,100);
  for (int i=5;i<=ran;i=i+5){
    servo1.write(i);
    delay(6*speed);
  }
  //check if player pulled their hands back
  if(!inRange()){
    pointComputer();
  }
  //reset position
  delay(500);
  for (int j=ran-3;j>=0;j=j-3){
    servo1.write(j);
    delay(40);
  }
  servo1.write(0);
}

void fullturn(int speed){
  //start from 0, make full 180 turn at game speed, check for hands, return to 0
  
  //full turn slightly faster than feint
  for (int i=0;i<=180;i=i+5){
    servo1.write(i);
    delay(3*speed);
  }
  //check for players hands
  if(inRange()){
    pointComputer();
  }
  else{
    pointPlayer();
  }
  //reset position
  delay(500);
  for (int j=180;j>0;j=j-3){
    servo1.write(j);
    delay(40);
  }
  servo1.write(0);
}

bool inRange(){
//checks for the players hands, returns true if their hands are in place
  digitalWrite(trigger,LOW);
  delayMicroseconds(5);
  digitalWrite(trigger,HIGH);
  delayMicroseconds(10);
  digitalWrite(trigger,LOW);
  dist=pulseIn(echo,HIGH); //returns length of pulse in microseconds (dist/58 for cm)
  Serial.println(dist/58);
  return dist/58 <= outofrange;
}

void updateScoring(int p_score,int comp_score){
  //change LEDs to match the current score
  if (comp_score==1){
    digitalWrite(11,HIGH);
    digitalWrite(12,LOW);
    digitalWrite(13,LOW);
  }
  else if (comp_score==2){
    digitalWrite(11,HIGH);
    digitalWrite(12,HIGH);
    digitalWrite(13,LOW);
  }
  else if (comp_score==3){
    digitalWrite(11,HIGH);
    digitalWrite(12,HIGH);
    digitalWrite(13,HIGH);
    gameOver();
  }
    
  if (p_score==1){
    digitalWrite(4,HIGH);
    digitalWrite(5,LOW);
    digitalWrite(6,LOW);
  }
  else if (p_score==2){
    digitalWrite(4,HIGH);
    digitalWrite(5,HIGH);
    digitalWrite(6,LOW);
  }
  else if (p_score>2){
    digitalWrite(4,HIGH);
    digitalWrite(5,HIGH);
    digitalWrite(6,HIGH);
    gameOver();
  }
}

void pointComputer(){
  //increase computer score by 1, change LEDs
  computerscore++;
  updateScoring(playerscore,computerscore);
}

void pointPlayer(){
  //increase player score by 1, change LEDs
  playerscore++;
  updateScoring(playerscore,computerscore);
}

void gameOver(){
  // play LED game over sequence, reset scores
  digitalWrite(11,HIGH);
  digitalWrite(12,HIGH);
  digitalWrite(13,HIGH);
  digitalWrite(4,HIGH);
  digitalWrite(5,HIGH);
  digitalWrite(6,HIGH);
  delay(2000);
  digitalWrite(11,LOW);
  digitalWrite(12,LOW);
  digitalWrite(13,LOW);
  digitalWrite(4,LOW);
  digitalWrite(5,LOW);
  digitalWrite(6,LOW);
  if (playerscore<computerscore){
    for (int i=1;i<=5;i++){
      digitalWrite(13,HIGH);
      digitalWrite(12,LOW);
      digitalWrite(11,HIGH);
      delay(250);
      digitalWrite(13,LOW);
      digitalWrite(12,HIGH);
      digitalWrite(11,LOW);
      delay(250);
    }
  }
  else{
    for (int i=1;i<=5;i++){
      digitalWrite(6,HIGH);
      digitalWrite(5,LOW);
      digitalWrite(4,HIGH);
      delay(250);
      digitalWrite(6,LOW);
      digitalWrite(5,HIGH);
      digitalWrite(4,LOW);
      delay(250);
    }
  }
  digitalWrite(11,LOW);
  digitalWrite(12,LOW);
  digitalWrite(13,LOW);
  digitalWrite(4,LOW);
  digitalWrite(5,LOW);
  digitalWrite(6,LOW);
  for (int i=1;i<=5;i++){
    // light chase victory
    digitalWrite(13,HIGH);
    delay(250);
    digitalWrite(12,HIGH);
    digitalWrite(13,LOW);
    delay(250);
    digitalWrite(11,HIGH);
    digitalWrite(12,LOW);
    delay(250);
    digitalWrite(4,HIGH);
    digitalWrite(11,LOW);
    delay(250);
    digitalWrite(5,HIGH);
    digitalWrite(4,LOW);
    delay(250);
    digitalWrite(6,HIGH);
    digitalWrite(5,LOW);
    delay(250);
    digitalWrite(6,LOW);
  }
  playerscore = 0;
  computerscore = 0;
  
}
